package net.knarcraft.knargui.item;

import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.OfflinePlayer;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.SkullMeta;
import org.bukkit.profile.PlayerProfile;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Base64;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * An item factory for generating player head-based icons
 */
@SuppressWarnings("unused")
public class PlayerHeadGUIItemFactory extends AbstractGUIItemFactory<PlayerHeadGUIItemFactory> {

    private static final String textureBaseURL = "https://textures.minecraft.net/texture/";
    private static final Pattern texturePattern = Pattern.compile("/texture/([0-9a-z]+)");

    /**
     * Instantiates a player head gui item factory
     */
    public PlayerHeadGUIItemFactory() {
        super(new ItemStack(Material.PLAYER_HEAD, 1));
        setChild(this);
    }

    /**
     * Uses the head skin from the given player
     *
     * @param player <p>The player whose skin should be used</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    public PlayerHeadGUIItemFactory useSkin(OfflinePlayer player) {
        return changeItemMeta(SkullMeta.class, (meta) -> meta.setOwningPlayer(player));
    }

    /**
     * Uses the head skin from the given player profile
     *
     * @param playerProfile <p>The player profile whose skin should be used</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    public PlayerHeadGUIItemFactory useSkin(PlayerProfile playerProfile) {
        return changeItemMeta(SkullMeta.class, (meta) -> meta.setOwnerProfile(playerProfile));
    }

    /**
     * Uses the head skin from the given player UUID
     *
     * @param uuid <p>The UUID of the skin to use</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    public PlayerHeadGUIItemFactory useSkin(UUID uuid) {
        return useSkin(Bukkit.getOfflinePlayer(uuid));
    }

    /**
     * Uses the skin from the given texture id
     *
     * <p>This methods takes the id after "<a href="https://textures.minecraft.net/texture/">
     * https://textures.minecraft.net/texture/</a>" as input. A base64 JSON value can also be used, though it's not
     * recommended.</p>
     *
     * @param textureId <p>The id of the texture to use</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    public PlayerHeadGUIItemFactory useSkin(String textureId) {
        //Get the texture id from a Base64 encoded JSON string
        if (!textureId.isBlank() && textureId.length() % 4 == 0) {
            String decoded = new String(Base64.getDecoder().decode(textureId));
            Matcher matcher = texturePattern.matcher(decoded);
            if (matcher.find()) {
                textureId = matcher.group(1);
            }
        }

        try {
            return useSkin(new URL(textureBaseURL + textureId));
        } catch (MalformedURLException exception) {
            throw new IllegalArgumentException("Texture id resulted in invalid URL");
        }
    }

    /**
     * Uses the head skin from the given URL
     *
     * @param url <p>The URL to use</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    public PlayerHeadGUIItemFactory useSkin(URL url) {
        PlayerProfile profile = Bukkit.createPlayerProfile(UUID.randomUUID());
        profile.getTextures().setSkin(url);
        return changeItemMeta(SkullMeta.class, (meta) -> meta.setOwnerProfile(profile));
    }

}
